<?php
namespace Controller;

use Service\Notification as NotificationService;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;

class Notification extends Simple {

    const NOTIFICATION_TITLE = 'Zmart';
    const EMAIL_OF_INDICATOR = "indicator";
    const EMAIL_OF_DATASOURCE = "dataSource";
    const EMAIL_OF_GOALS_NOTIFICATIONS = "goalNotifications";
    protected $notificationService;

    public function __construct(NotificationService $notificationService) {
        $this->notificationService = $notificationService;
    }

    protected function sendMailNotification($data, &$response, $type){
        try {
            $this->notificationService->sendMailNotification($data, $type);
            $response->addMessage(new Message("Mail sent successfully", "S"));
        } catch (\Exception $e) {
            $response->addMessage(new Message("Error on sending mail notification", "E"));
        }
    }

    public function sendNotificationDataSourceMail(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();

        $data = [
            "message"         => self::getVarFromRow('message', $row),
            "dataSourceName"  => self::getVarFromRow('dataSourceName', $row),
            "name"            => self::getVarFromRow('name', $row),
            "mail"            => self::getVarFromRow('email', $row),
            "to"              => self::getVarFromRow('name', $row)
        ];

        $this->sendMailNotification($data, $response, self::EMAIL_OF_DATASOURCE);
    }

    public function sendNotificationIndicatorMail(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();

        $data = [
            "message"           => self::getVarFromRow('message', $row),
            "sendDate"          => self::getVarFromRow('sendDate', $row),
            "nameIndicator"     => self::getVarFromRow('nameIndicator', $row),
            "indicator"         => self::getVarFromRow('indicator', $row),
            "sender"            => self::getVarFromRow('sender', $row),
            "to"                => self::getVarFromRow('to', $row),
            "messageType"       => self::getVarFromRow('messageType', $row),
            "indicatorPath"     => self::getVarFromRow('indicatorPath', $row)
        ];

        $this->sendMailNotification($data, $response, self::EMAIL_OF_INDICATOR);
        $this->sendPushNotification($data["message"], $data["indicatorPath"], $response);
    }

    public function sendNotificationGoalsMail(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();

        $data = [
            "labelView"            => self::getVarFromRow('label_view', $row),
            "linkIndicado"         => self::getVarFromRow('link_indicado', $row),
            "dataSource"            => self::getVarFromRow('datasource', $row),
            "lastUpdate"           => self::getVarFromRow('last_update', $row),
            "labelGoal"            => self::getVarFromRow('label_goal', $row),
            "goal"                  => self::getVarFromRow('goal', $row),
            "value"                 => self::getVarFromRow('value', $row),
            "userResponsible"      => self::getVarFromRow('user_responsible', $row),
            "notificationText"      => self::getVarFromRow('notificationText', $row),
            "to"                    => self::getVarFromRow('name', $row)
        ];

        $this->sendMailNotification($data, $response, self::EMAIL_OF_GOALS_NOTIFICATIONS);
    }

    protected function sendPushNotification($message, $indicatorPath, &$response){
        try {
            $this->notificationService->sendNotification(self::NOTIFICATION_TITLE, $message, $indicatorPath);
            $response->addMessage(new Message("Push notification sent successfully", "S"));
        } catch (\Exception $e) {
            $response->addMessage(new Message("Error on sending push notification", "E"));
        }
    }

    protected function getVarFromRow($varName, $row){
        return isset($row[$varName]) ? $row[$varName] : null;
    }
}