<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class PeriodCompare extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_PERIODS_LIST_COMPARE = "574f230498ba51386fab2a81";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getChangePercentage($value, $previousValue){
        $sign = $previousValue == $value ? "" : ($previousValue < $value ? "+" : "-");
        $number = $previousValue == 0 ? "∞%" : number_format(abs(100-($value/$previousValue*100)),2)."%";
        return ($previousValue === null) ? "" : ($sign.$number);
    }

    protected function getPeriodList($widget, $sort = false){
        $periodList = [];
        foreach($widget["period"] as $period){
            if (empty($period["start"]) && is_string($period)) {
                $periodFormatted = \Util\Period::decodeFormatted($period);
                $period = [
                    "start" => $periodFormatted[0],
                    "end" => $periodFormatted[1]
                ];
            }
            $periodList[strtotime($period["start"])] = implode("_",[$period["start"], $period["end"]]);
        }
        if ($sort){
            ksort($periodList);
        }
        return array_values($periodList);
    }

    protected function getMaskFunction($widget){
        return $widget["columns"][array_flip(array_column($widget["columns"], "name"))[$widget["periodsFact"]]]["mask"] ?? "";
    }

    protected function getPeriodCompareContainers($dataSourceDataList, $widget, $widgetIdPrefix = ""){
        $containers = [];
        $passedFirstRow = false;
        $index = 0;
        foreach($dataSourceDataList as $dataSourceData){
            $grid = [];
            $grid[] = [
                "name" => $widgetIdPrefix."row1_".$index,
                "component" => "ZdRow",
                "children" => [
                    [
                        "name" => $widgetIdPrefix."text_".$index,
                        "component" => "ZdText",
                        "text" => $dataSourceData[0]["label"],
                        "tag" => "h3",
                    ],
                ],
            ];
            $grid[] = [
                "name" => $widgetIdPrefix."rowGrid_".$index,
                "component" => "ZdRow",
                "cssStyle" => "overflow-x: hidden !important;",
                "children" => [
                    [
                        "name" => $widgetIdPrefix."colGrid_".$index,
                        "component" => "ZdCol",
                        "cssStyle" => "overflow-x: hidden !important;",
                        "children" => [
                            [
                                "name" => $widgetIdPrefix."grid_".$index,
                                "component" => "ZdGrid",
                                "fillWidth" => true,
                                "showTableHead" => false,
                                "showFooter" => false,
                                "showHeader" => false,
                                "toolbarSlot" => [],
                                "maskFunction" => $this->getMaskFunction($widget),
                                "cssStyle" => "min-width: min-content !important;",
                                "cssClass" => "periodCompareGrid",
                                "columns" => [
                                    [
                                        "name" => "period",
                                        "label" => "",
                                        "minWidth" => '150px',
                                        "maxWidth" => '150px',
                                        "width" => "150px",
                                        "cssStyle" => "width: 150px;",
                                        "cssClass" => "periodComparePeriodColumn",
                                        'align' => 'left',
                                    ],
                                    [
                                        "name" => "value",
                                        "label" => "",
                                        "minWidth" => '150px',
                                        "maxWidth" => '150px',
                                        "width" => "150px",
                                        'align' => 'right',
                                        "cssClass" => "zd-pr-2",
                                    ],
                                    [
                                        "name" => "changeIcon",
                                        "label" => "",
                                        "type" => "action",
                                        "minWidth" => '50px',
                                        "maxWidth" => '50px',
                                        'align' => 'left',
                                        "cssClass" => "zd-pa-0 zd-ma-0",
                                        "children" => [
                                            [
                                                "name" => $widgetIdPrefix."changeAction_".$index,
                                                "component" => "ZdButton",
                                                "icon" => true,
                                                "fab" => false,
                                                "flat" => false,
                                                "round" => false,
                                                'align' => 'left',
                                                "cssClass" => "zd-pa-0 zd-ma-0",
                                                "conditions" => [
                                                    "isVisible" => "{{ChartViewController.periodCompareChangeActionGetVisibility}}",
                                                    "color" => "{{ChartViewController.periodCompareChangeActionGetColor}}",
                                                    "iconName" => "{{ChartViewController.periodCompareChangeActionGetIconName}}",
                                                    "label" => "{{ChartViewController.periodCompareChangeActionGetLabel}}",
                                                ],
                                            ]
                                        ]
                                    ]
                                ],
                                "datasource" => [
                                    "uniqueKey" => "id",
                                    "data" => $dataSourceData
                                ]
                            ]
                        ]
                    ]
                ],
            ];
            $containers[] = [
                "component" => "zdContainer",
                "name" => $widgetIdPrefix."gridContainer_".$index,
                "cssClass" => "zd-pa-0 zd-pt-3 zd-pl-1 zd-ma-0",
                "cssStyle" => "min-width: 35%;",
                "children" => $grid,
            ];
            $passedFirstRow = true;
            $index++;
        }
        return $containers;
    }

    public function getChildren(){
        $data = [];
        $widget = $this->widget;
        $periodList = $this->getPeriodList($widget);
        $dataSourceDataList = [];
        $dataId = 0;
        foreach($this->widgetData as $widgetData){
            $dataSourceData = [];
            $previousValue = null;
            foreach($periodList as $period){
                $index = $widget["periodsListFact"] . "_" . $period;
                $value = !empty($widgetData[$index]) ? $widgetData[$index] : 0;
                $change = $this->getChangePercentage($value, $previousValue);
                $dataSourceData[] = [
                    "period" => str_replace("_"," - ",$period),
                    "value" => strval($value),
                    "change" => $change,
                    "changeValue" => floatval($change),
                    "infinity" => ($previousValue == 0),
                    "label" => $widgetData["zmartCustomDimension"],
                    "id" => $dataId,
                ];
                $previousValue = $value;
                $dataId++;
            }
            $dataSourceDataList[] = $dataSourceData;
        }
        usort($dataSourceDataList, function($a, $b) use ($widget){
            $valueA = $a[count($a)-1]["changeValue"];
            $valueB = $b[count($b)-1]["changeValue"];
            if(!empty($widget["zmartOrdenationComparePeriods"]) && strtolower($widget["zmartOrdenationComparePeriods"]) == "asc"){
                return $valueA == $valueB ? 0 : ($valueA > $valueB ? -1 : 1);
            }
            return $valueA == $valueB ? 0 : ($valueA < $valueB ? -1 : 1);
        });
        $widgetIdPrefix = $widget["_id"] . "_";
        $containers = $this->getPeriodCompareContainers($dataSourceDataList, $widget, $widgetIdPrefix);
        if(!empty($widget["label"])){
            $data[] = [
                "name" => $widgetIdPrefix."title_text",
                "component" => "ZdText",
                "text" => $widget["label"],
                "cssStyle" => "color: var(--v-primary-base) !important;",
                "tag" => "h2",
            ];
        }
        $horizontalFlexClass = "";
        if(empty($widget["isChartView"])){
            $horizontalFlexClass = "zd-display-flex zd-flex-row ";
        }
        $data[] = [
            "component" => "zdContainer",
            "name" => $widgetIdPrefix."periodCompareContainer",
            "cssClass" => $horizontalFlexClass."zd-pa-0 zd-ma-0",
            "cssStyle" => "min-width: 90vw !important; max-width: 100vw !important;",
            "children" => $containers,
        ];
        return $data;
    }


    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "viewTypeId" => self::TYPE_ID_PERIODS_LIST_COMPARE,
            "component" => "zdContainer",
            "children" => $this->getChildren(),
            "widget" => $widget,
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
