<?php
namespace Service;

use Service\Manager\Mongo\Manager;
use Util\Period;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Dashboard;
use \Service\Structure;
use \Service\Hierarchy;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Moment\Moment;

class ImportDefault {

    protected $environment;
    protected $manager;
    protected $dashboardService;

    protected $structureTypesOldIdList = [];


    public function __construct(Dashboard $dashboardService, Indicator $indicatorService, Manager $organizationManager,
            Structure $structureService, Hierarchy $hierarchyService)  {
        $this->dashboardService = $dashboardService;
        $this->indicatorService = $indicatorService;
        $this->organizationManager = $organizationManager;
        $this->structureService = $structureService;
        $this->hierarchyService = $hierarchyService;
    }

    const MENU_TREE_ID = "1";
    const DASHBOARD_TREE_ID = "2";

    protected $PARAMETERS_TO_CLEAN_BEFORE_IMPORT = [
		'parentTreeId',
		'primaryTreeId',
		'_breadcrumb',
		'_childCount',
        '_level',
        '_opened',
        '_parent',
        'alreadyImported'
    ];

    public function findImportIndicatorViews($conditions){
        $filterCriteria = new FilterCriteria("widget", $conditions);
        //  $filterCriteria->addCondition("indicator", $indicator);
        $widgets = $this->dashboardService->getDataFromMultipleOrgs($filterCriteria);
        return $widgets;
    }

    public function setLogImport($dashboardImported,$indicatorImported,$dataSourceImported){

        /**
         * @var $dataSourceService \Service\DataSource'
         * @var $viewService \Service\View'
         * @var $widgetService \Service\Widget'
         */
        $dataSourceService = InstanceManager::getInstance()->getService('\Service\DataSource');
        $viewService = InstanceManager::getInstance()->getService('\Service\View');
        $widgetService = InstanceManager::getInstance()->getService('\Service\Widget');

        foreach ($dashboardImported as $dashboard) {
            $viewService->setLog(null,$dashboard);
        }

        foreach ($indicatorImported as $indicator) {
            $this->indicatorService->setLog(null,$indicator);
            $widgets = $widgetService->findWidgetsByIndicator($indicator);
            foreach ($widgets as $widget) {
                $widgetService->setLog(null,$widget['_id']);
            }
        }
        foreach ($dataSourceImported as $dataSource) {
            $dataSourceService->setLog(null,$dataSource);
        }
    }

    public function findImportData($conditions) {
        $filterParentId = Utilities::getValueInFilterDataSourceByName($conditions, 'parentTreeId', true);
        $filterParentId = $filterParentId == -1 ? false : $filterParentId;
        
        $importData = array(
            array(
                "name" => "Menu",
                "primaryTreeId" => self::MENU_TREE_ID,
                "parentTreeId" => null,
                "type" => null
            ),
            array(
                "name" => "Dashboard",
                "primaryTreeId" => self::DASHBOARD_TREE_ID,
                "parentTreeId" => null,
                "type" => null
            )
        );
        
        if(empty($filterParentId) || $filterParentId != "NULL") {
            $importData = array();
        }

        $filterCriteria = new FilterCriteria("view", $conditions);
        $views = $this->dashboardService->getDataFromMultipleOrgs($filterCriteria);
        $viewPrimaryList=array();
        foreach($views as $i => &$view){
            if($view["type"] == "M"){
                if(empty($view["parent"])){
                    $view["parentTreeId"] = self::MENU_TREE_ID;
                } else {
                    $view["parentTreeId"] = self::MENU_TREE_ID."_".$view["parent"];
                }
                $view["primaryTreeId"] = self::MENU_TREE_ID."_".$view["_id"];
            } else if ($view["type"] == "D"){
                $view["parentTreeId"] = self::DASHBOARD_TREE_ID;
                $view["primaryTreeId"] = self::DASHBOARD_TREE_ID."_".$view["_id"];
                if(!empty($view["indicators"])){
                    foreach($view["indicators"] as $indicator){
                        $viewPrimaryList[$indicator["indicator"]][] = $view["primaryTreeId"];
                        $viewPrimaryList[$indicator["indicator"]] = array_unique($viewPrimaryList[$indicator["indicator"]]);
                    }
                }
            } else {
                unset($views[$i]);
            }
            if(!empty($filterParentId) && $view["parentTreeId"] != $filterParentId) {
                unset($views[$i]);
            }
        }
        $views = array_values($views);
        $this->setObjectsAlreadyImported($views,"view");
        
        $filterType = Utilities::getValueInFilterDataSourceByName($conditions, 'type', true);

        $filterCriteria = new FilterCriteria("indicator", $conditions);
        $indicators = $this->dashboardService->getDataFromMultipleOrgs($filterCriteria);
        $duplicatedIndicators = [];
        foreach($indicators as $i => &$indicator){
            $indicator["type"] = "I";
            if(!empty($indicator["menu"])){
                $indicator["parentTreeId"] = self::MENU_TREE_ID."_".$indicator["menu"];
            }
            if(!empty($viewPrimaryList[$indicator["_id"]])){
                if(!empty($indicator["menu"])){
                    $newIndicator = $indicator;
                    $newIndicator["menu"] = null;
                    $newIndicator["menuName"] = null;
                    $newIndicator["importOptions"] = "dashboardOnly";
                    $indicator["importOptions"] = "menuOnly";
                    $indicators[] = $newIndicator;
                } else {
                    foreach($viewPrimaryList[$indicator["_id"]] as $i => $primary){
                        if($i==0){
                            $indicator["parentTreeId"] = $primary;
                        } else {
                            $newIndicator = $indicator;
                            $newIndicator["parentTreeId"] = $primary;
                            $duplicatedIndicators[] = $newIndicator;
                        }
                    }
                }
            } else if(empty($indicator["menu"])){
                unset($indicators[$i]);
            }
            if(!empty($indicator)){
                if(empty($indicator["parentTreeId"])){
                    unset($indicators[$i]);
                } else {
                    $indicator["primaryTreeId"] = $indicator["parentTreeId"]."_".$indicator["_id"];
                }
            }
        }
        $indicators = array_merge($indicators,$duplicatedIndicators);
        foreach($indicators as $i => &$indicator){
            if(!empty($filterParentId) && $indicator["parentTreeId"] != $filterParentId) {
                unset($indicators[$i]);
            }
        }
        
        if(!empty($filterType)) {
            if((is_array($filterType) && !in_array("I", $filterType)) || $filterType == "I")
                $indicators = array();
        }
        $indicators = array_values($indicators);
        $this->setObjectsAlreadyImported($indicators,"indicator");

        $allImported = $this->setMainOptionsIfImportedEverything($importData,$views,$indicators);
        $importData = array_merge($importData, $views, $indicators);
        return $importData;
    }

    protected function setMainOptionsIfImportedEverything(&$importData,$views,$indicators){
        $allImported = array();
        $allImported["D"] = false;
        $allImported["M"] = false;
        $notImportedSuffix = "_notImported";
        $indicatorsDashboardList = [];
        $menuList = [];
        foreach($views as $view){
            if($view["type"]=="D"){
                foreach($view["indicators"] as $indicator){
                    $indicatorsDashboardList[] = $indicator["indicator"];
                }
            } else if($view["type"]=="M"){
                $menuList[] = $view["_id"];
            }
            if(!empty($view["alreadyImported"]) && empty($allImported[$view["type"].$notImportedSuffix])){
                $allImported[$view["type"]] = true;
            } else {
                $allImported[$view["type"]] = false;
                $allImported[$view["type"].$notImportedSuffix] = true;
            }
        }
        foreach($indicators as $indicator){
            if($allImported["D"] && empty($allImported["D".$notImportedSuffix])){
                if(in_array($indicator["_id"],$indicatorsDashboardList)){
                    if(empty($indicator["alreadyImported"])){
                        $allImported["D"] = false;
                        $allImported["D".$notImportedSuffix] = true;
                    }
                }
            }
            if($allImported["M"] && empty($allImported["M".$notImportedSuffix])){
                if(!empty($indicator["menu"]) && in_array($indicator["menu"],$menuList)){
                    if(empty($indicator["alreadyImported"])){
                        $allImported["M"] = false;
                        $allImported["M".$notImportedSuffix] = true;
                    }
                }
            }
        }
        foreach($importData as &$data){
            if($data["primaryTreeId"] == self::MENU_TREE_ID){
                $data["alreadyImported"] = ($allImported["M"] && empty($allImported["M".$notImportedSuffix]));
            } else {
                $data["alreadyImported"] = ($allImported["D"] && empty($allImported["D".$notImportedSuffix]));
            }
        }
    }

    protected function setObjectsAlreadyImported(&$objects,$dataSourceName){
        if (empty($objects)) {return;}
		$alreadyOnDb = $this->organizationManager->findBy(new FilterCriteria($dataSourceName,array()))->getRows();
        foreach($objects as &$object){
			foreach($alreadyOnDb as $dbObject){
                $object["alreadyImported"] = false;
                if(!empty($dbObject['oldId']) && $object['_id'] === $dbObject['oldId']){
                    $object["alreadyImported"] = true;
                    break;
                }
			}
		}
    }

    protected function cleanBeforeImport(&$object){
		foreach($this->PARAMETERS_TO_CLEAN_BEFORE_IMPORT as $parameter){
			if(isset($object[$parameter])){
				unset($object[$parameter]);
			}
		}
    }

    protected function getMenusAlreadyOnDb(){
        $viewsAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("view",array()))->getRows();
        $menuAlreadyOnDbList = [];
        foreach($viewsAlreadyOnDb as $row){
            if(!empty($row["type"])){
                if($row["type"] == "M"){
                    $menuAlreadyOnDbList[] = $row;
                }
            }
        }
        return $menuAlreadyOnDbList;
    }

    protected function getDrillDownIndicators($indicatorIdList){
        $indicatorIdListWithoutOptions = [];
        foreach($indicatorIdList as $indicatorId){
            $indicatorIdListWithoutOptions[] = explode("|",$indicatorId)[0];
        }
        return $this->indicatorService->checkDrillDownIndicators($indicatorIdListWithoutOptions);
    }

    protected function manualImportDataSource($dataSourceId){
        $dataSource = $this->organizationManager->findBy(new FilterCriteria("dataSource",
            array(array("columnName"=>"_id", "value" => $dataSourceId, "operator" => "="))
        ))->getRows()[0];
        if(!empty($dataSource["hasDataMining"])){
            $importSchedule = [];
            $importSchedule["dataSource"] = $dataSource["_id"];
            $importSchedule["persistType"] = "dropByPeriod";
            $importSchedule["scheduleExpression"] = "";
            $importSchedule["beforeImportScript"] = "";
            $importSchedule["afterImportScript"] = "";
            $importSchedule["status"] = "queue";
            $importSchedule["type"] = "manualBI";
            $importSchedule["date"] = new \MongoDB\BSON\UTCDateTime((new Moment(null, 'America/Sao_Paulo'))->startOf('day')->format('U').'000');
            if(is_array($dataSource["period"])){
                $importSchedule["period"] = $dataSource["period"];
            }else{
                list($importSchedule["period"]["start"],$importSchedule["period"]["end"]) = Period::decodeFormatted($dataSource["period"]);
            }
            $importScheduleService = InstanceManager::getInstance()->getService('\Service\ImportSchedule');
            $importScheduleService->saveOne($importSchedule);
        }
    }

    protected function findByOldId($oldId, $dataSourceName){
        return $this->organizationManager->findBy(new FilterCriteria($dataSourceName,
            array(array("columnName"=>"oldId", "value" => $oldId, "operator" => "="))
        ))->getRows();
    }

    public function importDefault($rows, $accessProfiles){
        $response = array();
        $error = array();
        $menuList = array();
        $indicatorIdList = [];
        $widgetIdList = [];
        $dashboardList = array();
        foreach($rows as $row){
            $this->cleanBeforeImport($row);
            if(!empty($row["type"])){
                if($row["type"] == "M"){
                    $menuList[] = $row;
                } else if($row["type"] == "D"){
                    $dashboardList[] = $row;
                } else if($row["type"] == "I"){
                    $inDashboard = array_search($row["_id"]."|dashboardOnly",$indicatorIdList);
                    $inMenu = array_search($row["_id"]."|menuOnly",$indicatorIdList);
                    if($inDashboard !== false || $inMenu !== false){
                        $index = $inDashboard !== false ? $inDashboard : $inMenu;
                        $indicatorIdList[$index] = explode("|",$indicatorIdList[$index])[0];
                    } else {
                        $indicatorId = $row["_id"];
                        if(!empty($row["importOptions"])){
                            $indicatorId.="|".$row["importOptions"];
                        }
                        $indicatorIdList[] = $indicatorId;
                    }
                }
            }
        }
        $menuAlreadyOnDbList = $this->getMenusAlreadyOnDb();
        $menuOldIdList = [];
        $response["menu"] = [];
        foreach($menuList as $menu){
            $oldId = $menu["_id"];
            $menu["oldId"] = $oldId;
            unset($menu["_id"]);

            $menu["accessProfiles"] = $accessProfiles;
            $parentMenu = !empty($menu["parent"]) ? $menu["parent"] : null;
            unset($menu["parent"]);

            $alreadyOnDb = $this->findByOldId($oldId, "view");
            if(!empty($alreadyOnDb) && !empty($alreadyOnDb[0])){
                $menu["_id"] = $alreadyOnDb[0]["_id"];
            }

            $persistedMenu = $this->dashboardService->saveDefaultOrgMenuOrgView($menu);
            if(empty($alreadyOnDb)){
                $response["menu"][] = $persistedMenu[0]["_id"];
            }
            $newId = $persistedMenu[0]["_id"];
            $menuOldIdList[$oldId] = $newId;
            if(!empty($parentMenu)){
                if(!empty($menuOldIdList[$parentMenu])){
                    $menu["_id"] = $newId;
                    $menu["parent"] = $menuOldIdList[$parentMenu];
                    $this->dashboardService->saveDefaultOrgMenuOrgView($menu);
                } else {
                    foreach($menuAlreadyOnDbList as $menuAlreadyOnDb){
                        if(!empty($menuAlreadyOnDb["oldId"]) && $menuAlreadyOnDb["oldId"] == $parentMenu){
                            $menu["_id"] = $newId;
                            $menu["parent"] = $menuAlreadyOnDb["_id"];
                            $this->dashboardService->saveDefaultOrgMenuOrgView($menu);
                            break;
                        }
                    }
                }
            }
        }

        $structureTypesAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("structureType",array()))->getRows();
        $this->structureTypesOldIdList = [];
        if(empty($structureTypesAlreadyOnDb)){
            try{
                $this->importDefaultStructures();
            }catch(\Exception $e){
                $error[] = $e->getMessage();
            }
            try{
                $this->importDefaultHierarchies();
            }catch(\Exception $e){
                $error[] = $e->getMessage();
            }
        }

        $newWidgets = [];
        $response["indicator"] = [];
        $response["widget"] = [];
        $response["dataSource"] = [];

        $drillDownIndicators = $this->getDrillDownIndicators($indicatorIdList);
        $indicatorIdList = array_merge($indicatorIdList,$drillDownIndicators);

        foreach($indicatorIdList as $indicatorId){
            $importOptions = explode("|",$indicatorId);
            $indicatorId = $importOptions[0];
            $importOptions = !empty($importOptions[1]) ? $importOptions[1] : null;

            $alreadyOnDb = $this->findByOldId($indicatorId, "indicator");

            if(!empty($alreadyOnDb) && !empty($alreadyOnDb[0])){
                if($importOptions != "menuOnly"){
                    $indicatorOldIdList[$indicatorId] = $alreadyOnDb[0]["_id"];
                }

                $widgetsAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("widget",
                    array(array("columnName"=>"indicator", "value" => $alreadyOnDb[0]["_id"], "operator" => "="))
                ))->getRows();

                if(!empty($widgetsAlreadyOnDb)){
                    foreach($widgetsAlreadyOnDb as $widget){
                        if(!empty($widget) && !empty($widget["oldId"])){
                            $widgetIdList[$widget["oldId"]] = $widget["_id"];
                        }
                    }
                }
            } else {
                $persistedIndicator = $this->indicatorService->saveDefaultOrgIndicatorsAndDataSources([$indicatorId], true, $menuOldIdList, $importOptions, $this->structureTypesOldIdList);
                if(!empty($persistedIndicator["indicator"]) && $importOptions != "menuOnly"){
                    $response["indicator"][] = $persistedIndicator["indicator"][0]["_id"];
                    $indicatorOldIdList[$indicatorId] = $persistedIndicator["indicator"][0]["_id"];
                }
                if(!empty($persistedIndicator["newDatasource"])){
                    $dataSourceAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("dataSource",
                        array(array("columnName"=>"oldId", "value" => $persistedIndicator["newDatasource"][0]["_id"], "operator" => "="))
                    ))->getRows();
                    if(empty($dataSourceAlreadyOnDb)){
                        if(!in_array($persistedIndicator["newDatasource"][0]["_id"],$response["dataSource"])){
                            $response["dataSource"][] = $persistedIndicator["newDatasource"][0]["_id"];
                        }
                    }
                }
                if(!empty($persistedIndicator["widget"])){
                    for($i = 0; $i < sizeof($persistedIndicator["widget"]); $i+=2){
                        if(!empty($persistedIndicator["widget"][$i+1])){
                            foreach($persistedIndicator["widget"][$i] as $j => $widget){
                                $response["widget"][] = $widget["_id"];
                                $widgetIdList[$persistedIndicator["widget"][$i+1][$j]] = $widget["_id"];
                                $newWidgets[] = $widget["_id"];
                            }
                        }
                    }
                    $widgetsAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("widget",array()))->getRows();
                    foreach($widgetsAlreadyOnDb as $widget){
                        if(in_array($widget["_id"],$newWidgets)){
                            if(!empty($widget["hasDrillDown"]) && !empty($widget["drillDownView"]) && !empty($widgetIdList[$widget["drillDownView"]])){
                                $widget["drillDownView"] = $widgetIdList[$widget["drillDownView"]];
                            } else {
                                $widget["hasDrillDown"] = false;
                                $widget["drillDownView"] = null;
                                $widget["drillDownViewField"] = null;
                            }
                            $this->indicatorService->saveDefaultOrgViews([$widget]);
                        }
                    }
                }
            }
        }

        if(!empty($response["dataSource"]) && is_array($response["dataSource"])){
            $importScheduleService = InstanceManager::getInstance()->getService('\Service\ImportSchedule');
            foreach($response["dataSource"] as $dataSource){
                try{
                    $existingImportSchedule = $importScheduleService->findBy(new FilterCriteria("importSchedule",
                        array(array("columnName"=>"dataSource", "value" => $dataSource, "operator" => "="))
                    ));
                    if(empty($existingImportSchedule)){
                        $this->manualImportDataSource($dataSource);
                    }
                }catch(\Exception $e){
                    $error[] = $e->getMessage();
                }
            }
        }

        $response["dashboard"] = [];
        foreach($dashboardList as $dashboard){
            $oldId = $dashboard["_id"];
            $dashboard["oldId"] = $oldId;
            unset($dashboard["_id"]);

            $alreadyOnDb = $this->findByOldId($oldId, "view");

            if(!empty($alreadyOnDb) && !empty($alreadyOnDb[0])){
                $dashboard["_id"] = $alreadyOnDb[0]["_id"];
                if(!empty($alreadyOnDb[0]["indicators"])){
                    $dashboard["originalIndicators"] = $alreadyOnDb[0]["indicators"];
                }
            }

            if(!empty($accessProfiles)){
				$dashboard["accessProfiles"] = $accessProfiles;
			} else {
                unset($dashboard["accessProfiles"]);
            }
            if(!empty($dashboard["indicators"])){
                foreach($dashboard["indicators"] as $i => &$indicator){
                    if(empty($indicatorOldIdList[$indicator["indicator"]])){
                        unset($dashboard["indicators"][$i]);
                    } else {
                        $indicator["oldId"] = $indicator["indicator"];
                        $indicator["indicator"] = $indicatorOldIdList[$indicator["indicator"]];
                        if(!empty($indicator["widget"]) && !empty($widgetIdList[$indicator["widget"]])){
                            $indicator["widget"] = $widgetIdList[$indicator["widget"]];
                        } else {
                            unset($dashboard["indicators"][$i]);
                        }
                    }
                    $this->indicatorService->importPeriod($indicator);
                }
                $dashboard["indicators"] = array_values($dashboard["indicators"]);
                if(!empty($dashboard["originalIndicators"])){
                    foreach($dashboard["indicators"] as $i=>$indicator){
                        foreach($dashboard["originalIndicators"] as $originalIndicator){
                            if(!empty($originalIndicator["oldId"]) && !empty($indicator["oldId"]) && $originalIndicator["oldId"]==$indicator["oldId"]){
                                unset($dashboard["originalIndicators"][$i]);
                                break;
                            }
                        }
                    }
                    $dashboard["indicators"] = array_merge($dashboard["originalIndicators"],array_values($dashboard["indicators"]));
                    unset($dashboard["originalIndicators"]);
                }
            }
            $persistedDashboard = $this->dashboardService->saveDefaultOrgMenuOrgView($dashboard);
            if(empty($alreadyOnDb)){
                $response["dashboard"][] = $persistedDashboard[0]["_id"];
            }
        }

        return array($response,$error);
    }

    protected function importDefaultHierarchies(){
        $structureTypes = $this->organizationManager->findBy(new FilterCriteria("structureType",array()))->getRows();
        $structureTypesById = array_flip(array_column($structureTypes,"_id"));
        $hierarchyTypes = $this->dashboardService->getDataFromMultipleOrgs(new FilterCriteria("hierarchyType"));
        foreach($structureTypes as $structureType){
            $this->structureTypesOldIdList[$structureType["oldId"]] = $structureType["_id"];
        }
        foreach($hierarchyTypes as &$hierarchyType){
            $hierarchyType["oldId"] = $hierarchyType["_id"];
            unset($hierarchyType["_id"]);
            if(empty($this->structureTypesOldIdList[$hierarchyType["typeParent"]]) || empty($this->structureTypesOldIdList[$hierarchyType["typeChild"]])){
                break;
            }
            $hierarchyType["typeParent"] = $this->structureTypesOldIdList[$hierarchyType["typeParent"]];
            $hierarchyType["typeChild"] = $this->structureTypesOldIdList[$hierarchyType["typeChild"]];
        }
        $this->organizationManager->persist(new DataSet('hierarchyType', $hierarchyTypes));
        $hierarchyTypes = $this->organizationManager->findBy(new FilterCriteria("hierarchyType",array()))->getRows();
        $hierarchyTypesOldIdList = [];
        foreach($hierarchyTypes as $hierarchyType){
            $hierarchyTypesOldIdList[$hierarchyType["oldId"]] = $hierarchyType["_id"];
        }
        $hierarchies = [];
        foreach($hierarchyTypes as &$hierarchyType){
            if(!empty($hierarchyType["parent"]) && !empty($hierarchyTypesOldIdList[$hierarchyType["parent"]])){
                $hierarchyType["parent"] = $hierarchyTypesOldIdList[$hierarchyType["parent"]];
            }
            try{
                $parentStructureType = $structureTypes[$structureTypesById[$hierarchyType["typeParent"]]];
                $childStructureType = $structureTypes[$structureTypesById[$hierarchyType["typeChild"]]];
                $fieldsKeyRelation = [];
                $intersectKeys = array_intersect($parentStructureType["fieldKey"],$childStructureType["fieldKey"]);
                foreach($intersectKeys as $key){
                    $fieldKeysRelation[] = array("childKey"=>$key,"parentKey"=>$key);
                }
                $parentStructures = $this->organizationManager->findBy(new FilterCriteria("structure",
                    array(array("columnName"=>"type", "value" => $hierarchyType["typeParent"], "operator" => "="))
                ))->getRows();
                $childStructures = $this->organizationManager->findBy(new FilterCriteria("structure",
                    array(array("columnName"=>"type", "value" => $hierarchyType["typeChild"], "operator" => "="))
                ))->getRows();
                try{
                    foreach($parentStructures as $parentStructure) {
                        $hierarchies = array_merge($hierarchies, $this->hierarchyService->generateHierarchy($parentStructure, $childStructures, $hierarchyType["_id"], $fieldKeysRelation));
                    }
                }catch(\Exception $e){
                    throw $e;
                }
            }catch(\Exception $e){

            }
        }
        if(!empty($hierarchies)){
            $this->organizationManager->persist(new DataSet('hierarchy', $hierarchies));
        }
        $this->organizationManager->persist(new DataSet('hierarchyType', $hierarchyTypes));
    }

    protected function importDefaultStructures(){
        $filterCriteria = new FilterCriteria("structureType");
        $structureTypes = $this->dashboardService->getDataFromMultipleOrgs($filterCriteria);
        list($connection,$connectionName) = $this->indicatorService->getMainConnection();

        foreach($structureTypes as $i => &$rows){
            $rows["oldId"] = $rows["_id"];
            unset($rows["_id"]);
            $rows["connection"] = $connection;
        }
        $this->organizationManager->persist(new DataSet('structureType', $structureTypes));
        $structureTypes = $this->organizationManager->findBy(new FilterCriteria("structureType",array()))->getRows();
        $managerId = $this->structureService->getOrgManagerCdoperador();
        $parentlessHierarchyTypes = $this->dashboardService->getDataFromMultipleOrgs(new FilterCriteria("hierarchyType",
            array(array("columnName"=>"parent", "value" => null, "operator" => FilterCriteria::EQ))
        ));
        $parentlessHierarchyTypes = empty($parentlessHierarchyTypes) ? [] : array_column($parentlessHierarchyTypes, "typeParent");
        foreach($structureTypes as &$structureType){
            try{
                $structures = $this->structureService->loadStructuresFromQuery($structureType);
                foreach($structures as &$row){
                    if(isset($row['externalId'])) {
                        $row['externalId'] = Utilities::decodeStructureExternalId($row['externalId']);
                    }
                    if(isset($row['CDOPERADOR'])) {
                        unset($row['CDOPERADOR']);
                    }
                    if(isset($row['NMOPERADOR'])) {
                        unset($row['NMOPERADOR']);
                    }
                    if(in_array($structureType["oldId"],$parentlessHierarchyTypes)){
                        $row["usersResponsible"] = $managerId ? [$managerId] : [];
                    }else{
                        $row["usersResponsible"] = [];
                    }
                }
            }catch(\Exception $e){
                throw new \Exception(Exception::buildErrorJSON(
                    "NOT_CONNECTED",
                    array("connName"=>$connectionName,"testResult"=>$e->getMessage())
                ));
            }
            $persistedStructures = $this->organizationManager->persist(new DataSet('structure', $structures));
            foreach($persistedStructures as $persistedStructure){
                $structure = $this->structureService->findById($persistedStructure["_id"]);
                $this->structureService->saveLogStructure(null,[$structure],$persistedStructure["_id"],$structure["name"],$structure["type"]);
            }
        }
    }

}
