<?php
namespace Service;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Manager\Mongo\Manager;

class ViewType {

    protected $dataSourceName = 'viewType';
    /** @var \Service\Manager\Mongo\Manager $mongoManager */
    protected $mongoManager;
    /** @var \Service\ViewProperty $viewPropertyService */
    protected $viewPropertyService;

    public function __construct(Manager $mongoManager, ViewProperty $viewPropertyService) {
        $this->mongoManager = $mongoManager;
        $this->viewPropertyService = $viewPropertyService;
    }

    public function findBy(FilterCriteria $filterCriteria) {
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function findAll() {
        return $this->findBy(new FilterCriteria($this->dataSourceName));
    }

    public function findById($id) {

        $viewType = TemplateJson::getViewTypeById($id);
        if (empty($viewType) ) {
            return null;
        } else {
            if (isset($viewType['properties'])) {
               $viewType['properties'] =  TemplateJson::findViewPropertyByIds($viewType['properties']);
            }
            if (!empty($viewType['dimensionProperty'])) {
                $viewType['dimensionProperty'] = TemplateJson::findViewPropertyById($viewType['dimensionProperty']);
            }
            if (!empty($viewType['factProperties'])) {
                $viewType['factProperties'] = TemplateJson::findViewPropertyByIds($viewType['factProperties']);
            }
            if (isset($viewType['factProperty']) && !!$viewType['factProperty']) {
                $viewType['factProperty'] = TemplateJson::findViewPropertyById($viewType['factProperty']);
            }
            if (!empty($viewType['convertToColumns'])) {
                $viewType['convertToColumns'] = TemplateJson::findViewPropertyByIds($viewType['convertToColumns']);
                if(isset($viewType['convertToColumns'])){
                    usort($viewType['convertToColumns'], function($a, $b) {
                        return strcmp($a['name'],$b['name']);
                    });
                }
            }
            return $viewType;
        }
    }

    public function findAllWithDimensionAndFactProperties() {
        $types = $this->findAll();
        $properties = $this->viewPropertyService->findAll();
        $this->relateViewTypeProperties($types, $properties);
        return $types;
    }

    protected function relateViewTypeProperties(array &$types, array $properties) {
        $propertiesMap = array();
        foreach ($properties as $property) {
            $propertiesMap[$property['_id']] = $property;
        }

        foreach ($types as &$type) {
            if (isset($type['dimensionProperty'])) {
                $type['dimensionProperty'] = $propertiesMap[$type['dimensionProperty']];
            }
            if (isset($type['factProperties'])) {
                foreach ($type['factProperties'] as &$fact) {
                    $fact = $propertiesMap[$fact];
                }
            }
        }
    }

}