import {ChartViewInterface} from "@/Services/ChartView/ChartViewInterface";
import {ChartViewAbstract} from "@/Services/ChartView/ChartViewAbstract";
import {ChartViewService} from "@/Services/ChartViewService";
import {ZmartChartsApex} from "@/Services/Chats/ZmartChartsApex";
import {IFieldConditionsParam} from "@zeedhi/zd-crosstab-common";

export class Crosstab extends ChartViewAbstract implements ChartViewInterface {
    public auxAVG: any = {};

    public getWidget(dashboard: any, widget: any) {
        const height = widget[0].style;
        if ((dashboard.label !== '' || dashboard.label !== null) && widget[0].children && widget[0].children.length > 0) {
            widget[0].children[0].text = dashboard.label;
        }
        widget[0].style = "background: " + dashboard.color + "; " + height;
        return widget;
    }

    private setPeriodCompareFormatterFunctionsAndDrill(chart: any) {
        const widget = chart.widget;
        const searchFunction = (_chart: any) => {
            if (_chart.component && _chart.component === "ZdGrid") {
                if (_chart.maskFunction) {
                    _chart.datasource.data.forEach((element: any) => {
                        if (element.value) {
                            element.value = ChartViewService.getMaskFunction(_chart.maskFunction)(parseFloat(element.value));
                        }
                    });
                }
                if (widget.hasDrillDown) {
                    _chart.events = {
                        "rowClick": this.getZmartCharts().drillPeriodCompare.bind(this.getZmartCharts(), widget),
                    };
                }
            }
        };
        searchFunction(chart);
    }

    public setDrill(charts: any, zmartCharts: ZmartChartsApex) {
        this.zmartCharts = zmartCharts;
        charts[0].children[1].events = {
            "cellClick": this.getZmartCharts().drillCrosstab.bind(this.getZmartCharts(), charts[0].children[1].widget),
        };
        return charts;
    }
    getEditData(viewData: any) {
        viewData.xKeysYMultiple = viewData.xKeysY;
        return viewData;
    }

    public getSaveViewParams(formExtra: any, values: any, cleanValues: any) {
        const chartSeries = [
            {
                serieColor: [null],
                serieColors: values.serieColors || null,
                serieFields: values.serieFields || null,
                serieLabels: values.serieLabels || null,
                serieLineWidth: values.serieLineWidth || null,
                serieStacked: values.serieStacked || null,
                serieLineType: values.serieLineType || null,
                serieYAxisFormat: values.serieYAxisFormat || null,
                serieType: values.serieType || null
            }

        ];
        cleanValues.chartSeries = chartSeries;
        delete cleanValues.serieColors;
        cleanValues.yAxisProperty = cleanValues.xKeysYMultiple;
        cleanValues.xKeysY = cleanValues.xKeysYMultiple;
        cleanValues.xKeys = cleanValues.xKeysYMultiple;
        cleanValues.xKeys.push(cleanValues.xKeysX);
        return cleanValues;
    }

    protected applyStyles(gridStyles: any, column: any, row: any) {
        if (gridStyles) {
            let cssClasse: string = '';
            gridStyles.forEach((style: any) => {
                if (this.columnCompare(column, style.styleFields) && this.evalCondition(style.condition, row)) {
                    cssClasse = style.cssClass;
                }
            });
            return cssClasse;
        }
        return {};
    }

    protected columnCompare(column: any, styles: any) {
        if (Array.isArray(column)) {
            return column.some((element: any) => styles.includes(element));
        }
        return styles.includes(column);
    }

    public conditionsCustomCssStyles(gridStyle: any, cellName: any, eventNext: any) {
        const row:any = {};
        row[cellName] = eventNext.cell.value;
        return this.applyStyles(gridStyle, cellName, row);
    }

    public conditionsCustomCssStylesRow(gridStyle: any, cellName: any, eventNext: any) {
        const row:any = {};
        row[cellName] = eventNext.field.value;
        return this.applyStyles(gridStyle, cellName, row);
    }

    public crossAggregation(widget: any, aggregation: string, sum: number, row: any) {
        let value = sum;
        if (row[widget.cell.name] || row[widget.cell.name] === 0) {
            value = row[widget.cell.name];
            if (sum === null) {
                sum = value;
            }
        }
        if (aggregation === 'MAX') {
            if (value > sum) {
                sum = value;
            }
        } else if (aggregation === 'MiN') {
            if (value < sum) {
                sum = value;
            }
        } else if (aggregation === 'SUM') {
                sum += value;
        }
        return sum;
    }

    public checkAppliedCustomCss(styleName: string): boolean {
        return !!document.getElementById("Zmart_custom_" + styleName);
    }

    public addcss(styleName: string, cssCode: string) {
        if (!this.checkAppliedCustomCss(styleName)) {
            const style = document.createElement('style');
            style.type = 'text/css';
            style.innerHTML = cssCode;
            style.id = "Zmart_custom_" + styleName;
            document.getElementsByTagName('head')[0].appendChild(style);
        }
    }

    public getCharts(charts: any) {
        if (charts[0].children[1].mask !== null) {
            charts[0].children[1].cell.formatter = ChartViewService.getMaskFunction(charts[0].children[1].mask);
        }
        if (charts[0].children[1].widget.gridStyle) {
            charts[0].children[1].widget.gridStyle.forEach((style: any) => {
                this.addcss(style.styleName, style.css.cssCode);
            });
            charts[0].children[1].cell.conditions.cssClass = this.conditionsCustomCssStyles.bind(this, charts[0].children[1].widget.gridStyle, charts[0].children[1].cell.name);
            charts[0].children[1].rows.forEach((row: any) => {
                row.conditions.cssClass = this.conditionsCustomCssStylesRow.bind(this, charts[0].children[1].widget.gridStyle, row.name);
            });
        }
        charts[0].children[1].cell.reducer = this.crossAggregation.bind(this, charts[0].children[1], charts[0].children[1].cell.reducer);

        return charts;
    }
}
