import { Loader, Metadata } from '@zeedhi/core';
import { Modal, ModalService, DialogService, LoadingService, Container, Icon, Progress, Text } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { DataSourceService } from '@/Services/DataSourceService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceController } from './DataSourceController';

export class DSImportHistoryController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");
    public service = new DataSourceService();

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    private importHistoryModal!: Modal;
    public importHistoryData: any = {};
    public async openImportHistoryModalModal({ row }: any) {
        this.importHistoryModal?.destroy();
        this.importHistoryModal = await ModalService.createFromJson('dataSource/importHistory/importHistoryLog', true);
        this.importHistoryData = row || {};
        this.importHistoryModal.show();
    }
    public closeImportHistoryModal() {
        this.importHistoryModal.hide();
    }
    public importHistoryLogOnMounted({ component }: any) {
        const importHistoryLog = component.getChildInstance("importHistoryLog");
        importHistoryLog.value = this.importHistoryData.logs?.map((el: any) => `Type: ${el.type}\nMessage: ${el.message}`).join("\n\n");
    }
    public deleteImportHistory(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/importHistory/remove", "importHistory-grid");
    }
    protected getDateString(date: any) {
        const day = String(date.getDate()).padStart(2, '0');
        const month = String(date.getMonth() + 1).padStart(2, '0');
        const year = date.getFullYear();
        return [day, month, year].join("/");
    }
    protected getDefaultDateRange(date: any) {
        const today = this.getDateString(date);
        date.setDate(date.getDate() - 3);
        const previousDay = this.getDateString(date);
        return [previousDay, today];
    }
    public updateImportHistoryFilter() {
        if (Metadata.getInstances('importHistory-grid').length === 0 || !Metadata.getInstances('importHistory-grid')[0]) {
            return;
        }
        const dataSourceId = this.datasourceData._id;
        const importHistoryGrid: any = Metadata.getInstance<TekGrid>("importHistory-grid");
        importHistoryGrid.datasource.dynamicFilter = {
            dataSourceId: [
              {
                operation: "EQUALS",
                relation: "AND",
                value: dataSourceId,
              }
            ],
            createDate: [
               {
                operation: "BETWEEN",
                relation: "AND",
                value: this.getDefaultDateRange(new Date()),
               }
            ]
          };
        importHistoryGrid.datasource.order = ["endDate.desc"];
        importHistoryGrid.datasource.get();
    }
    public updateProgressBarDataSourceGrid({row}: any) {
        const importStatus = {
            status: row.importStatus,
            importHasError: row.importHasError,
            _id: row._id,
            importSubprocesses: row.importSubprocesses,
            totalImported: row.totalImported
        };
        this.updateProgressBar({row: importStatus});
    }
    public updateProgressBar({row}: any) {
        setTimeout(() => {
            const barComponent = Metadata.getInstance<Progress>("importProgressBar_" + row._id);
            const iconComponent = Metadata.getInstance<Icon>("importProgressIcon_" + row._id);
            const textComponent = Metadata.getInstance<Text>("importProgressText_" + row._id);

            const status = row.status;
            const subprocesses = row.importSubprocesses || row.subprocesses || 0;
            const totalImported = row.totalImported || 0;

            if (row.importHasError || row.hasError) {
                iconComponent.isVisible = true;
                iconComponent.iconName = "cancel";
                iconComponent.color = "red";
            } else if (status === 's') {
                textComponent.isVisible = true;
                textComponent.text = "Saving documents...";
            } else if (status === 100) {
                iconComponent.isVisible = true;
                iconComponent.iconName = "success";
                iconComponent.color = "green";
            } else if (status === 'c') {
                iconComponent.isVisible = true;
                iconComponent.iconName = "cancel";
                iconComponent.color = "orange";
            } else if ((status > 0 && status < 100) || (status >= 0 && status < 100 && subprocesses > 0)) {
                barComponent.isVisible = true;
                barComponent.value = parseFloat(status);
                barComponent.centerSlot[0].text = status + "%";
            } else if (totalImported > 0) {
                textComponent.isVisible = true;
                textComponent.text = `In process (${totalImported} linhas)...`;
            } else if (status === 0 && subprocesses === 0) {
                textComponent.isVisible = true;
                textComponent.text = "Starting processes...";
            }
        }, 0);
    }
}
