import { Metadata } from '@zeedhi/core';
import { SelectTree } from '@zeedhi/common';

export class FilterModalController {
    public onMountedFilterModal() {
        const filterStructure = localStorage.getItem('filterStructure');
        if (filterStructure) {
            const structureField = Metadata.getInstance<SelectTree>("structure_label");
            structureField.setValue(filterStructure);
            localStorage.setItem('filterStructure', "");
        }
    }

    public getChartViewFilterModalJson(filterModalParameters: any) {
        localStorage.setItem('filterStructure', filterModalParameters.selectedStructure);
        let period = filterModalParameters.period ? filterModalParameters.period : null;
        if (Array.isArray(period) && period[0]) {
            period = period[0];
        }
        const filterFields: any = [
            {
               "name": "structure_label",
               "component": "ZdSelectTree",
               "label": "Estruturas Gerenciais",
               "dataValue": "_id",
               "dataText": "name",
               "parentField": "parent",
               "grid": {
                  "cols": 12
               },
               "value": filterModalParameters.selectedStructure,
               "datasource": {
                 "uniqueKey": "_id",
                 "data": filterModalParameters.structures,
             }
            },
            {
               "name": "dimensionFilter",
               "component": filterModalParameters.hasMultipleDimensions ? "ZdSelectMultiple" : "ZdSelect",
               "label": filterModalParameters.hasMultipleDimensions ? "Dimensões" : "Dimensão",
               "grid": {
                   "cols": 12
               },
               "dataText": "name",
               "dataValue": "id",
               "value": filterModalParameters.selectedDimension,
               "datasource": {
                 "data": filterModalParameters.dimensions
             }
            },
            {
               "name": "factsFilter",
               "component": filterModalParameters.hasMultipleFacts ? "ZdSelectMultiple" : "ZdSelect",
               "label": filterModalParameters.hasMultipleFacts ? "Fatos" : "Fato",
               "grid": {
                  "cols": 12
               },
               "returnObject": false,
               "dataText": "name",
               "dataValue": "id",
               "value": filterModalParameters.selectedFacts,
               "datasource": {
                 "data": filterModalParameters.facts
              }
            },
            {
               "name": "drillDownFilters",
               "component": "ZdSelectMultiple",
               "label": "DrillDown Filters",
               "grid": {
                  "cols": 12
               },
               "dataText": "name",
               "dataValue": "value",
               "returnObject": true,
               "value": filterModalParameters.drillDownFilters,
               "datasource": {
                 "data": filterModalParameters.drillDownFilters
             }
            },
            {
               "name": "filterByDimension",
               "component": "ZdSelectMultiple",
               "label": filterModalParameters.selectedDimension,
               "isVisible": false,
               "grid": {
                  "cols": 12
               },
               "datasource": {
                   "type": "simple",
                   "items": filterModalParameters.dimensionData ? filterModalParameters.dimensionData : null
               }
            },
            {
               "name": "basicUsageDivider",
               "component": "ZdDivider"
            },
            {
               "name": "period",
               "component": "ZdDateRange",
               "splitter": " até ",
               "displayFormat": "DD/MM/YYYY",
               "dateFormat": "DD/MM/YYYY",
               "inputFormat": "DD/MM/YYYY",
               "label": "Periodo",
               "value": period,
               "grid": {
                  "cols": 12
               },
               "helperOptions": ["CURRENT_YEAR"]
            }
        ];
        if (filterModalParameters.dimensions) {
            filterFields.push({
                "name": "basicUsageDivider",
                "component": "ZdDivider"
            });
            filterModalParameters.dimensions.forEach((dimension: any) => {
                if (filterModalParameters.dimensionFilterValues[dimension.id]) {
                    const selectedValue = filterModalParameters.drillDownFilters.find((filter: any) => filter.name === dimension.id);
                    filterFields.push({
                        "name": "filter_" + dimension.id,
                        "component": "ZdSelectMultiple",
                        "label": dimension.name,
                        "grid": {
                            "cols": 12
                        },
                        "value": selectedValue ? selectedValue.value : null,
                        "datasource": {
                            "type": "simple",
                            "items": filterModalParameters.dimensionFilterValues[dimension.id]
                        }
                    });
                }
            });
        }
        return {
            "name": "addStructureType",
            "persistent": true,
            "children": [
               {
                  "name": "backStructureTypeModal",
                  "component": "ZdHeader",
                  "color": "transparent",
                  "padless": true,
                  "elevation": 0,
                  "leftSlot": [
                     {
                        "name": "structureTypeName",
                        "component": "ZdText",
                         "tag": "h3",
                         "text": "Filtro"
                     }
                  ],
                  "rightSlot": [
                     {
                        "name": "closeFilterModal",
                        "component": "ZdButton",
                        "small": true,
                        "icon": true,
                        "flat": true,
                        "iconName": "close",
                        "events": {
                           "click": "{{ChartViewController.closeFilterModal}}"
                        }
                     }
                  ]
               },
               {
                  "name": "filterForm",
                  "component": "ZdForm",
                  "cssClass": "zd-my-2",
                  "cssStyle": "max-height: 450px !important",
                  "events": {
                    "onMounted": "{{FilterModalController.onMountedFilterModal}}"
                  },
                  "children": filterFields,
               },
               {
                  "name": "changeClienteModal",
                  "component": "ZdFooter",
                  "color": "transparent",
                  "padless": true,
                  "rightSlot": [
                     {
                        "name": "cancelFilterModal",
                        "component": "ZdButton",
                        "label": "Cancelar",
                        "outline": true,
                        "events": {
                           "click": "{{ChartViewController.closeFilterModal}}"
                        }
                     },
                     {
                        "name": "SAVE",
                        "component": "ZdButton",
                        "label": "OK",
                        "events": {
                         "click": "{{ChartViewController.applyFilter}}"
                      }
                     }
                  ]
               }
            ]
          };
    }
}
